# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime, timedelta

import dateutil.parser
import pytz

DEFAULT_VOLUME_TYPE = "my_block_storage_type"
DEFAULT_VOLUME_NAME = "my_block_storage_name"


def get_instance_create_end_sample(instance_id=None, tenant_id=None, flavor_name=None,
                                   creation_timestamp=None, name=None, os_distro=None, os_version=None, metadata={}):
    kwargs = {
        "instance_id": instance_id or "e7d44dea-21c1-452c-b50c-cbab0d07d7d3",
        "tenant_id": tenant_id or "0be9215b503b43279ae585d50a33aed8",
        "hostname": name or "to.to",
        "display_name": name or "to.to",
        "instance_type": flavor_name or "myflavor",
        "os_distro": os_distro or "CentOS",
        "os_version": os_version or "6.4",
        "created_at": creation_timestamp if creation_timestamp else datetime(2014, 2, 14, 16, 29, 58, tzinfo=pytz.utc),
        "launched_at": creation_timestamp + timedelta(seconds=1) if creation_timestamp else datetime(2014, 2, 14, 16,
                                                                                                     30, 02,
                                                                                                     tzinfo=pytz.utc),
        "terminated_at": None,
        "deleted_at": None,
        "state": "active",
        "metadata": metadata
    }
    kwargs["timestamp"] = kwargs["launched_at"] + timedelta(microseconds=200000)
    return _get_instance_payload("compute.instance.create.end", **kwargs)


def get_instance_delete_end_sample(instance_id=None, tenant_id=None, flavor_name=None, os_distro=None, os_version=None,
                                   creation_timestamp=None, deletion_timestamp=None, name=None):
    kwargs = {
        "instance_id": instance_id,
        "tenant_id": tenant_id,
        "hostname": name,
        "display_name": name,
        "instance_type": flavor_name,
        "os_distro": os_distro or "centos",
        "os_version": os_version or "6.4",
        "created_at": creation_timestamp if creation_timestamp else datetime(2014, 2, 14, 16, 29, 58, tzinfo=pytz.utc),
        "launched_at": creation_timestamp + timedelta(seconds=1) if creation_timestamp else datetime(2014, 2, 14, 16,
                                                                                                     30, 02,
                                                                                                     tzinfo=pytz.utc),
        "terminated_at": deletion_timestamp if deletion_timestamp else datetime(2014, 2, 18, 12, 5, 23,
                                                                                tzinfo=pytz.utc),
        "deleted_at": deletion_timestamp if deletion_timestamp else datetime(2014, 2, 18, 12, 5, 23, tzinfo=pytz.utc),
        "state": "deleted"
    }
    kwargs["timestamp"] = kwargs["terminated_at"] + timedelta(microseconds=200000)
    return _get_instance_payload("compute.instance.delete.end", **kwargs)


def get_volume_create_end_sample(volume_id=None, tenant_id=None, volume_type=None, volume_size=None,
                                 creation_timestamp=None, name=None):
    kwargs = {
        "volume_id": volume_id or "64a0ca7f-5f5a-4dc5-a1e1-e04e89eb95ed",
        "tenant_id": tenant_id or "46eeb8e44298460899cf4b3554bfe11f",
        "display_name": name or "mytenant-0001-myvolume",
        "volume_type": volume_type or DEFAULT_VOLUME_TYPE,
        "volume_size": volume_size or 50,
        "created_at": creation_timestamp if creation_timestamp else datetime(2014, 2, 14, 17, 18, 35, tzinfo=pytz.utc),
        "launched_at": creation_timestamp + timedelta(seconds=1) if creation_timestamp else datetime(2014, 2, 14, 17,
                                                                                                     18, 40,
                                                                                                     tzinfo=pytz.utc),
        "status": "available"
    }
    kwargs["timestamp"] = kwargs["launched_at"] + timedelta(microseconds=200000)
    return _get_volume_icehouse_payload("volume.create.end", **kwargs)


def get_volume_delete_end_sample(volume_id=None, tenant_id=None, volume_type=None, volume_size=None,
                                 creation_timestamp=None, deletion_timestamp=None, name=None):
    kwargs = {
        "volume_id": volume_id or "64a0ca7f-5f5a-4dc5-a1e1-e04e89eb95ed",
        "tenant_id": tenant_id or "46eeb8e44298460899cf4b3554bfe11f",
        "display_name": name or "mytenant-0001-myvolume",
        "volume_type": volume_type or DEFAULT_VOLUME_TYPE,
        "volume_size": volume_size or 50,
        "created_at": creation_timestamp if creation_timestamp else datetime(2014, 2, 14, 17, 18, 35, tzinfo=pytz.utc),
        "launched_at": deletion_timestamp if deletion_timestamp else datetime(2014, 2, 14, 17, 18, 40, tzinfo=pytz.utc),
        "timestamp": deletion_timestamp if deletion_timestamp else datetime(2014, 2, 23, 8, 1, 58, tzinfo=pytz.utc),
        "status": "deleting"
    }
    return _get_volume_icehouse_payload("volume.delete.end", **kwargs)


def get_volume_attach_icehouse_end_sample(volume_id=None, tenant_id=None, volume_type=None, volume_size=None,
                                          creation_timestamp=None, name=None, attached_to=None):
    kwargs = {
        "volume_id": volume_id or "64a0ca7f-5f5a-4dc5-a1e1-e04e89eb95ed",
        "tenant_id": tenant_id or "46eeb8e44298460899cf4b3554bfe11f",
        "display_name": name or "mytenant-0001-myvolume",
        "volume_type": volume_type or DEFAULT_VOLUME_TYPE,
        "volume_size": volume_size or 50,
        "attached_to": attached_to or "e7d44dea-21c1-452c-b50c-cbab0d07d7d3",
        "created_at": creation_timestamp if creation_timestamp else datetime(2014, 2, 14, 17, 18, 35, tzinfo=pytz.utc),
        "launched_at": creation_timestamp + timedelta(seconds=1) if creation_timestamp else datetime(2014, 2, 14, 17,
                                                                                                     18, 40,
                                                                                                     tzinfo=pytz.utc),
        "timestamp": creation_timestamp + timedelta(seconds=1) if creation_timestamp else datetime(2014, 2, 14, 17, 18,
                                                                                                   40, tzinfo=pytz.utc),
    }
    return _get_volume_icehouse_payload("volume.attach.end", **kwargs)


def get_volume_attach_kilo_end_sample(volume_id=None, tenant_id=None, volume_type=None, volume_size=None,
                                      timestamp=None, name=None, attached_to=None):
    kwargs = {
        "volume_id": volume_id or "64a0ca7f-5f5a-4dc5-a1e1-e04e89eb95ed",
        "tenant_id": tenant_id or "46eeb8e44298460899cf4b3554bfe11f",
        "display_name": name or "mytenant-0001-myvolume",
        "volume_type": volume_type or DEFAULT_VOLUME_TYPE,
        "volume_size": volume_size or 50,
        "attached_to": attached_to,
        "timestamp": timestamp + timedelta(seconds=1) if timestamp else datetime(2014, 2, 14, 17, 18, 40,
                                                                                 tzinfo=pytz.utc),
    }
    return _get_volume_kilo_payload("volume.attach.end", **kwargs)


def get_volume_detach_kilo_end_sample(volume_id=None, tenant_id=None, volume_type=None, volume_size=None,
                                      timestamp=None, name=None, attached_to=None):
    kwargs = {
        "volume_id": volume_id or "64a0ca7f-5f5a-4dc5-a1e1-e04e89eb95ed",
        "tenant_id": tenant_id or "46eeb8e44298460899cf4b3554bfe11f",
        "display_name": name or "mytenant-0001-myvolume",
        "volume_type": volume_type or DEFAULT_VOLUME_TYPE,
        "volume_size": volume_size or 50,
        "attached_to": attached_to,
        "timestamp": timestamp + timedelta(seconds=1) if timestamp else datetime(2014, 2, 14, 17, 18, 40,
                                                                                 tzinfo=pytz.utc),
    }
    return _get_volume_kilo_payload("volume.detach.end", **kwargs)


def get_volume_resize_end_sample(volume_id=None, tenant_id=None, volume_type=None, volume_size=None,
                                 timestamp=None, name=None, attached_to=None):
    kwargs = {
        "volume_id": volume_id or "64a0ca7f-5f5a-4dc5-a1e1-e04e89eb95ed",
        "tenant_id": tenant_id or "46eeb8e44298460899cf4b3554bfe11f",
        "display_name": name or "mytenant-0001-myvolume",
        "volume_type": volume_type or DEFAULT_VOLUME_TYPE,
        "volume_size": volume_size or 50,
        "attached_to": attached_to,
        "timestamp": timestamp + timedelta(seconds=1) if timestamp else datetime(2014, 2, 14, 17, 18, 40,
                                                                                 tzinfo=pytz.utc),
    }
    return _get_volume_kilo_payload("volume.resize.end", **kwargs)


def get_volume_detach_end_sample(volume_id=None, tenant_id=None, volume_type=None, volume_size=None,
                                 creation_timestamp=None, deletion_timestamp=None, name=None):
    kwargs = {
        "volume_id": volume_id or "64a0ca7f-5f5a-4dc5-a1e1-e04e89eb95ed",
        "tenant_id": tenant_id or "46eeb8e44298460899cf4b3554bfe11f",
        "display_name": name or "mytenant-0001-myvolume",
        "volume_type": volume_type or DEFAULT_VOLUME_TYPE,
        "volume_size": volume_size or 50,
        "attached_to": None,
        "created_at": creation_timestamp if creation_timestamp else datetime(2014, 2, 14, 17, 18, 35, tzinfo=pytz.utc),
        "launched_at": creation_timestamp + timedelta(seconds=1) if creation_timestamp else datetime(2014, 2, 14, 17,
                                                                                                     18, 40,
                                                                                                     tzinfo=pytz.utc),
        "timestamp": deletion_timestamp if deletion_timestamp else datetime(2014, 2, 23, 8, 1, 58, tzinfo=pytz.utc),
        "status": "detach"
    }
    return _get_volume_icehouse_payload("volume.detach.end", **kwargs)


def get_volume_rename_end_sample(volume_id=None, tenant_id=None, volume_type=None, volume_size=None,
                                 creation_timestamp=None, deletion_timestamp=None, name=None):
    kwargs = {
        "volume_id": volume_id or "64a0ca7f-5f5a-4dc5-a1e1-e04e89eb95ed",
        "tenant_id": tenant_id or "46eeb8e44298460899cf4b3554bfe11f",
        "display_name": name or "mytenant-0001-mysnapshot01",
        "volume_type": volume_type or DEFAULT_VOLUME_TYPE,
        "volume_size": volume_size or 50,
        "attached_to": None,
        "created_at": creation_timestamp if creation_timestamp else datetime(2014, 2, 14, 17, 18, 35, tzinfo=pytz.utc),
        "launched_at": creation_timestamp + timedelta(seconds=1) if creation_timestamp else datetime(2014, 2, 14, 17,
                                                                                                     18, 40,
                                                                                                     tzinfo=pytz.utc),
        "timestamp": deletion_timestamp if deletion_timestamp else datetime(2014, 2, 23, 8, 1, 58, tzinfo=pytz.utc),
        "status": "detach"
    }
    return _get_volume_icehouse_payload("volume.update.end", **kwargs)


def get_volume_exists_sample(volume_id=None, tenant_id=None, volume_type=None, volume_size=None,
                             creation_timestamp=None, deletion_timestamp=None, name=None):
    kwargs = {
        "volume_id": volume_id or "64a0ca7f-5f5a-4dc5-a1e1-e04e89eb95ed",
        "tenant_id": tenant_id or "46eeb8e44298460899cf4b3554bfe11f",
        "display_name": name or "mytenant-0001-mysnapshot",
        "volume_type": volume_type or DEFAULT_VOLUME_TYPE,
        "volume_size": volume_size or 50,
        "attached_to": None,
        "created_at": creation_timestamp if creation_timestamp else datetime(2014, 2, 14, 17, 18, 35, tzinfo=pytz.utc),
        "launched_at": creation_timestamp + timedelta(seconds=1) if creation_timestamp else datetime(2014, 2, 14, 17,
                                                                                                     18, 40,
                                                                                                     tzinfo=pytz.utc),
        "timestamp": deletion_timestamp if deletion_timestamp else datetime(2014, 2, 23, 8, 1, 58, tzinfo=pytz.utc),
        "status": "detach"
    }
    return _get_volume_icehouse_payload("volume.exists", **kwargs)


def _format_date(datetime_obj):
    return datetime_obj.strftime("%Y-%m-%dT%H:%M:%S.%fZ")


def _get_instance_payload(event_type, instance_id=None, tenant_id=None, hostname=None, display_name=None,
                          instance_type=None,
                          instance_flavor_id=None, timestamp=None, created_at=None, launched_at=None,
                          deleted_at=None, terminated_at=None, state=None, os_type=None, os_distro=None,
                          os_version=None, metadata={}):
    instance_id = instance_id or "e7d44dea-21c1-452c-b50c-cbab0d07d7d3"
    os_type = os_type or "linux"
    os_distro = os_distro or "centos"
    os_version = os_version or "6.4"
    hostname = hostname or "to.to"
    display_name = display_name or "to.to"
    tenant_id = tenant_id or "0be9215b503b43279ae585d50a33aed8"
    instance_type = instance_type or "myflavor"
    instance_flavor_id = instance_flavor_id or "201"
    timestamp = timestamp if timestamp else "2014-02-14T16:30:10.453532Z"
    created_at = _format_date(created_at) if created_at else "2014-02-14T16:29:58.000000Z"
    launched_at = _format_date(launched_at) if launched_at else "2014-02-14T16:30:10.221171Z"
    deleted_at = _format_date(deleted_at) if deleted_at else ""
    terminated_at = _format_date(terminated_at) if terminated_at else ""
    state = state or "active"
    metadata = metadata

    if not isinstance(timestamp, datetime):
        timestamp = dateutil.parser.parse(timestamp)

    return {
        "event_type": event_type,
        "payload": {
            "state_description": "",
            "availability_zone": None,
            "terminated_at": terminated_at,
            "ephemeral_gb": 0,
            "instance_type_id": 12,
            "message": "Success",
            "deleted_at": deleted_at,
            "memory_mb": 1024,
            "user_id": "2525317304464dc3a03f2a63e99200c8",
            "reservation_id": "r-7e68nhfk",
            "hostname": hostname,
            "state": state,
            "launched_at": launched_at,
            "node": "mynode.domain.tld",
            "ramdisk_id": "",
            "access_ip_v6": None,
            "disk_gb": 50,
            "access_ip_v4": None,
            "kernel_id": "",
            "image_name": "CentOS 6.4 x86_64",
            "host": "node02",
            "display_name": display_name,
            "root_gb": 50,
            "tenant_id": tenant_id,
            "created_at": created_at,
            "instance_id": instance_id,
            "instance_type": instance_type,
            "vcpus": 1,
            "image_meta": {
                "min_disk": "50",
                "container_format": "bare",
                "min_ram": "256",
                "disk_format": "qcow2",
                "build_version": "68",
                "version": os_version,
                "architecture": "x86_64",
                "auto_disk_config": "True",
                "os_type": os_type,
                "base_image_ref": "ea0d5e26-a272-462a-9333-1e38813bac7b",
                "distro": os_distro
            },
            "architecture": "x86_64",
            "os_type": "linux",
            "instance_flavor_id": instance_flavor_id,
            "metadata": metadata
        },
        "timestamp": timestamp.strftime("%Y-%m-%dT%H:%M:%S.%fZ"),
        "updated_at": _format_date(timestamp - timedelta(seconds=10)),
    }


def _get_volume_icehouse_payload(event_type, volume_id=None, tenant_id=None, display_name=None, volume_type=None,
                                 volume_size=None, timestamp=None, created_at=None, launched_at=None, status=None,
                                 attached_to=None):
    volume_id = volume_id or "64a0ca7f-5f5a-4dc5-a1e1-e04e89eb95ed"
    tenant_id = tenant_id or "46eeb8e44298460899cf4b3554bfe11f"
    display_name = display_name or "mytenant-0001-myvolume"
    volume_type = volume_type or DEFAULT_VOLUME_TYPE
    volume_size = volume_size or 50
    timestamp = timestamp if timestamp else "2014-02-14T17:18:40.888401Z"
    created_at = _format_date(created_at) if created_at else "2014-02-14T17:18:35.000000Z"
    launched_at = _format_date(launched_at) if launched_at else "2014-02-14T17:18:40.765844Z"
    status = status or "available"
    attached_to = attached_to or "e7d44dea-21c1-452c-b50c-cbab0d07d7d3"

    if not isinstance(timestamp, datetime):
        timestamp = dateutil.parser.parse(timestamp)

    return {
        "event_type": event_type,
        "timestamp": launched_at,
        "publisher_id": "volume.cinder01",
        "payload": {
            "instance_uuid": attached_to,
            "status": status,
            "display_name": display_name,
            "availability_zone": "nova",
            "tenant_id": tenant_id,
            "created_at": created_at,
            "snapshot_id": None,
            "volume_type": volume_type,
            "volume_id": volume_id,
            "user_id": "ebc0d5a5ecf3417ca0d4f8c90d682f6e",
            "launched_at": launched_at,
            "size": volume_size,
        },
        "priority": "INFO",
        "updated_at": _format_date(timestamp - timedelta(seconds=10)),

    }


def _get_volume_kilo_payload(event_type, volume_id=None, tenant_id=None, display_name=None, volume_type=None,
                             timestamp=None, attached_to=None, volume_size=1):
    volume_id = volume_id or "64a0ca7f-5f5a-4dc5-a1e1-e04e89eb95ed"
    tenant_id = tenant_id or "46eeb8e44298460899cf4b3554bfe11f"
    display_name = display_name or "mytenant-0001-myvolume"
    volume_type = volume_type or DEFAULT_VOLUME_TYPE
    timestamp = timestamp if timestamp else "2014-02-14T17:18:40.888401Z"
    attached_to = attached_to
    volume_attachment = []

    if not isinstance(timestamp, datetime):
        timestamp = dateutil.parser.parse(timestamp)

    if attached_to:
        for instance_id in attached_to:
            volume_attachment.append({
                "instance_uuid": instance_id,
                "attach_time": _format_date(timestamp - timedelta(seconds=10)),
                "deleted": False,
                "attach_mode": "ro",
                "created_at": _format_date(timestamp - timedelta(seconds=10)),
                "attached_host": "",
                "updated_at": _format_date(timestamp - timedelta(seconds=10)),
                "attach_status": 'available',
                "detach_time": "",
                "volume_id": volume_id,
                "mountpoint": "/dev/vdd",
                "deleted_at": "",
                "id": "228345ee-0520-4d45-86fa-1e4c9f8d057d"
            })

    return {
        "event_type": event_type,
        "timestamp": _format_date(timestamp),
        "publisher_id": "volume.cinder01",
        "payload": {
            "status": "in-use",
            "display_name": display_name,
            "volume_attachment": volume_attachment,
            "availability_zone": "nova",
            "tenant_id": tenant_id,
            "created_at": "2015-07-27T16:11:07Z",
            "volume_id": volume_id,
            "volume_type": volume_type,
            "host": "web@lvmdriver-1#lvmdriver-1",
            "replication_status": "disabled",
            "user_id": "aa518ac79d4c4d61b806e64600fcad21",
            "metadata": [],
            "launched_at": "2015-07-27T16:11:08Z",
            "size": volume_size
        },
        "priority": "INFO",
        "updated_at": _format_date(timestamp - timedelta(seconds=10)),
    }


def get_instance_rebuild_end_sample():
    return _get_instance_payload("compute.instance.rebuild.end")


def get_instance_resized_end_sample():
    return _get_instance_payload("compute.instance.resize.confirm.end")


def get_volume_update_end_sample(volume_id=None, tenant_id=None, volume_type=None, volume_size=None,
                                 creation_timestamp=None, deletion_timestamp=None, name=None):
    kwargs = {
        "volume_id": volume_id or "64a0ca7f-5f5a-4dc5-a1e1-e04e89eb95ed",
        "tenant_id": tenant_id or "46eeb8e44298460899cf4b3554bfe11f",
        "display_name": name or "mytenant-0001-myvolume",
        "volume_type": volume_type or DEFAULT_VOLUME_TYPE,
        "volume_size": volume_size or 50,
        "created_at": creation_timestamp if creation_timestamp else datetime(2014, 2, 14, 17, 18, 35, tzinfo=pytz.utc),
        "launched_at": deletion_timestamp if deletion_timestamp else datetime(2014, 2, 23, 8, 1, 58, tzinfo=pytz.utc),
        "timestamp": deletion_timestamp if deletion_timestamp else datetime(2014, 2, 23, 8, 1, 58, tzinfo=pytz.utc),
        "status": "deleting"
    }
    return _get_volume_icehouse_payload("volume.resize.end", **kwargs)


def get_volume_type_create_sample(volume_type_id, volume_type_name):
    return {
        "event_type": "volume_type.create",
        "publisher_id": "volume.cinder01",
        "payload": {
            "volume_types": {
                "name": volume_type_name,
                "qos_specs_id": None,
                "deleted": False,
                "created_at": "2014-02-14T17:18:35.036186Z",
                "extra_specs": {},
                "deleted_at": None,
                "id": volume_type_id,
            }
        },
        "updated_at": "2014-02-14T17:18:35.036186Z",
    }



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from flexmock import flexmock, flexmock_teardown
from hamcrest import instance_of, assert_that

from almanach import config
from almanach.adapters.auth_adapter import AuthenticationAdapter
from almanach.auth.keystone_auth import KeystoneAuthentication
from almanach.auth.mixed_auth import MixedAuthentication
from almanach.auth.private_key_auth import PrivateKeyAuthentication


class AuthenticationAdapterTest(unittest.TestCase):
    def tearDown(self):
        flexmock_teardown()

    def test_assert_that_the_default_backend_is_private_key(self):
        adapter = AuthenticationAdapter().factory()
        assert_that(adapter, instance_of(PrivateKeyAuthentication))

    def test_get_keystone_auth_backend(self):
        flexmock(config).should_receive("auth_strategy").and_return("keystone")
        adapter = AuthenticationAdapter().factory()
        assert_that(adapter, instance_of(KeystoneAuthentication))

    def test_get_mixed_auth_backend(self):
        flexmock(config).should_receive("auth_strategy").and_return("token,keystone")
        adapter = AuthenticationAdapter().factory()
        assert_that(adapter, instance_of(MixedAuthentication))



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from hamcrest import assert_that, equal_to
from base_api_testcase import BaseApiTestCase


class ApiAuthenticationTest(BaseApiTestCase):

    def test_list_entities_unauthorized(self):
        list_query = "{url}/project/{project}/instances?start={start}&end={end}"
        response = self.almanachHelper.get(url=list_query, headers={'Accept': 'application/json'},
                                           project="e455d65807cb4796bd72abecdc8a76ba",
                                           start="2014-02-28 18:50:00.000", end="2014-03-21 22:00:00.000")
        assert_that(response.status_code, equal_to(401))



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from retry import retry
from uuid import uuid4
from hamcrest import equal_to, assert_that, has_entry

from helpers.rabbit_mq_helper import RabbitMqHelper
from helpers.almanach_helper import AlmanachHelper


class BaseApiTestCase(unittest.TestCase):
    rabbitMqHelper = RabbitMqHelper()
    almanachHelper = AlmanachHelper()

    def _create_instance_entity(self):
        project_id = "my_test_project_id"
        instance_id = str(uuid4())
        data = {
            'id': instance_id,
            'created_at': '2016-01-01T18:30:00Z',
            'name': 'integration_test_instance_FlavorA',
            'flavor': 'FlavorA',
            'os_type': 'FreeBSD',
            'os_distro': 'Stable',
            'os_version': '10',
        }
        response = self.almanachHelper.post(url="{url}/project/{project}/instance", data=data, project=project_id)
        assert_that(response.status_code, equal_to(201))
        return instance_id

    @classmethod
    @retry(exceptions=AssertionError, delay=10, max_delay=300)
    def _wait_until_volume_type_is_created(cls, volume_type_id):
        assert_that(cls._get_volume_types(volume_type_id),
                    has_entry("volume_type_id", volume_type_id))

    @classmethod
    def _get_volume_types(cls, type_id):
        query = "{url}/volume_type/{type_id}"
        response = cls.almanachHelper.get(url=query, type_id=type_id)
        return response.json()



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import logging
import kombu

from kombu.mixins import ConsumerMixin
from almanach import config
from almanach.adapters.instance_bus_adapter import InstanceBusAdapter
from almanach.adapters.volume_bus_adapter import VolumeBusAdapter


class BusAdapter(ConsumerMixin):

    def __init__(self, controller, connection, retry_adapter):
        super(BusAdapter, self).__init__()
        self.controller = controller
        self.connection = connection
        self.retry_adapter = retry_adapter

    def on_message(self, notification, message):
        try:
            self._process_notification(notification)
        except Exception as e:
            logging.warning("Sending notification to retry letter exchange {0}".format(json.dumps(notification)))
            logging.exception(e.message)
            self.retry_adapter.publish_to_dead_letter(message)
        message.ack()

    def _process_notification(self, notification):
        if isinstance(notification, basestring):
            notification = json.loads(notification)

        event_type = notification.get("event_type")
        logging.info("Received event: '{0}'".format(event_type))
        InstanceBusAdapter(self.controller).handle_events(event_type, notification)
        VolumeBusAdapter(self.controller).handle_events(event_type, notification)

    def get_consumers(self, consumer, channel):
        queue = kombu.Queue(config.rabbitmq_queue(), routing_key=config.rabbitmq_routing_key())
        return [consumer(
            [queue],
            callbacks=[self.on_message],
            auto_declare=False)]

    def run(self, _tokens=1):
        try:
            super(BusAdapter, self).run(_tokens)
        except KeyboardInterrupt:
            pass



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import flask

from unittest import TestCase
from datetime import datetime
from flexmock import flexmock, flexmock_teardown

from almanach import config
from almanach.adapters import api_route_v1 as api_route
from almanach.common.exceptions.authentication_failure_exception import AuthenticationFailureException


class BaseApi(TestCase):
    def setUp(self):
        self.prepare()
        self.prepare_with_successful_authentication()

    def tearDown(self):
        flexmock_teardown()

    @staticmethod
    def having_config(key, value):
        (flexmock(config)
         .should_receive(key)
         .and_return(value))

    def prepare(self):
        self.controller = flexmock()
        self.auth_adapter = flexmock()
        api_route.controller = self.controller
        api_route.auth_adapter = self.auth_adapter

        self.app = flask.Flask("almanach")
        self.app.register_blueprint(api_route.api)

    def prepare_with_successful_authentication(self):
        self.having_config('auth_private_key', 'some token value')
        self.auth_adapter.should_receive("validate").and_return(True)

    def prepare_with_failed_authentication(self):
        self.having_config('auth_private_key', 'some token value')
        self.auth_adapter.should_receive("validate").and_raise(AuthenticationFailureException("Wrong credentials"))

    def api_get(self, url, query_string=None, headers=None, accept='application/json'):
        return self._api_call(url, "get", None, query_string, headers, accept)

    def api_head(self, url, headers):
        return self._api_call(url=url, method="head", headers=headers, accept='application/json')

    def api_post(self, url, data=None, query_string=None, headers=None, accept='application/json'):
        return self._api_call(url, "post", data, query_string, headers, accept)

    def api_put(self, url, data=None, query_string=None, headers=None, accept='application/json'):
        return self._api_call(url, "put", data, query_string, headers, accept)

    def api_delete(self, url, query_string=None, data=None, headers=None, accept='application/json'):
        return self._api_call(url, "delete", data, query_string, headers, accept)

    def _api_call(self, url, method, data=None, query_string=None, headers=None, accept='application/json'):
        with self.app.test_client() as http_client:
            if not headers:
                headers = {}
        headers['Accept'] = accept
        result = getattr(http_client, method)(url, data=json.dumps(data), query_string=query_string, headers=headers)
        return_data = json.loads(result.data) \
            if result.headers.get('Content-Type') == 'application/json' \
            else result.data
        return result.status_code, return_data


class DateMatcher(object):
    def __init__(self, date):
        self.date = date

    def __eq__(self, other):
        return other == self.date


def a_date_matching(date_string):
    return DateMatcher(datetime.strptime(date_string, "%Y-%m-%d %H:%M:%S.%f"))



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from uuid import uuid4
from datetime import datetime
from retry import retry

import pytz
from hamcrest import assert_that, equal_to, has_entry

from base_api_testcase import BaseApiTestCase
from builders.messages import get_instance_create_end_sample


class CollectorInstanceCreateTest(BaseApiTestCase):
    def test_instance_create(self):
        instance_id = str(uuid4())
        tenant_id = str(uuid4())

        self.rabbitMqHelper.push(
            get_instance_create_end_sample(
                    instance_id=instance_id,
                    tenant_id=tenant_id,
                    creation_timestamp=datetime(2016, 2, 1, 9, 0, 0, tzinfo=pytz.utc)
            ))

        self.assert_that_instance_entity_is_created(instance_id, tenant_id)

    @retry(exceptions=AssertionError, delay=1, max_delay=300)
    def assert_that_instance_entity_is_created(self, instance_id, tenant_id):
        entities = self.almanachHelper.get_entities(tenant_id, "2016-01-01 00:00:00.000")
        assert_that(len(entities), equal_to(1))
        assert_that(entities[0], has_entry("entity_id", instance_id))



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytz

from copy import copy
from datetime import datetime
from uuid import uuid4

from almanach.core.model import build_entity_from_dict, Instance, Volume, VolumeType


class Builder(object):

    def __init__(self, dict_object):
        self.dict_object = dict_object


class EntityBuilder(Builder):

    def build(self):
        return build_entity_from_dict(self.dict_object)

    def with_id(self, entity_id):
        self.dict_object["entity_id"] = entity_id
        return self

    def with_project_id(self, project_id):
        self.dict_object["project_id"] = project_id
        return self

    def with_last_event(self, last_event):
        self.dict_object["last_event"] = last_event
        return self

    def with_start(self, year, month, day, hour, minute, second):
        self.with_datetime_start(datetime(year, month, day, hour, minute, second, tzinfo=pytz.utc))
        return self

    def with_datetime_start(self, date):
        self.dict_object["start"] = date
        return self

    def with_end(self, year, month, day, hour, minute, second):
        self.dict_object["end"] = datetime(year, month, day, hour, minute, second, tzinfo=pytz.utc)
        return self

    def with_no_end(self):
        self.dict_object["end"] = None
        return self

    def with_flavor(self, flavor):
        self.dict_object["flavor"] = flavor
        return self

    def with_metadata(self, metadata):
        self.dict_object['metadata'] = metadata
        return self

    def build_from(self, other):
        self.dict_object = copy(other.__dict__)
        return self

    def with_all_dates_in_string(self):
        self.dict_object['start'] = self.dict_object['start'].strftime("%Y-%m-%dT%H:%M:%S.%fZ")
        self.dict_object['last_event'] = self.dict_object['last_event'].strftime("%Y-%m-%dT%H:%M:%S.%fZ")
        return self


class VolumeBuilder(EntityBuilder):

    def with_attached_to(self, attached_to):
        self.dict_object["attached_to"] = attached_to
        return self

    def with_no_attachment(self):
        self.dict_object["attached_to"] = []
        return self

    def with_display_name(self, display_name):
        self.dict_object["name"] = display_name
        return self

    def with_volume_type(self, volume_type):
        self.dict_object["volume_type"] = volume_type
        return self


class VolumeTypeBuilder(Builder):

    def build(self):
        return VolumeType(**self.dict_object)

    def with_volume_type_id(self, volume_type_id):
        self.dict_object["volume_type_id"] = volume_type_id
        return self

    def with_volume_type_name(self, volume_type_name):
        self.dict_object["volume_type_name"] = volume_type_name
        return self


def instance():
    return EntityBuilder({
        "entity_id": str(uuid4()),
        "project_id": str(uuid4()),
        "start": datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
        "end": None,
        "last_event": datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
        "flavor": "A1.1",
        "os": {
            "os_type": "windows",
            "distro": "windows",
            "version": "2012r2"
        },
        "entity_type": Instance.TYPE,
        "name": "some-instance",
        "metadata": {
            "a_metadata.to_filter": "include.this",
            "a_metadata.to_exclude": "exclude.this"
        }
    })


def volume():
    return VolumeBuilder({
        "entity_id": str(uuid4()),
        "project_id": str(uuid4()),
        "start": datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
        "end": None,
        "last_event": datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
        "volume_type": "SF400",
        "size": 1000000,
        "entity_type": Volume.TYPE,
        "name": "some-volume",
        "attached_to": None,
    })


def volume_type():
    return VolumeTypeBuilder({
        "volume_type_id": str(uuid4()),
        "volume_type_name": "a_type_name"
    })


def a(builder):
    return builder.build()



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from datetime import datetime

import pkg_resources
import mongomock
from flexmock import flexmock, flexmock_teardown
from hamcrest import assert_that, contains_inanyorder

from pymongo import MongoClient
import pytz

from almanach.adapters.database_adapter import DatabaseAdapter
from almanach.common.exceptions.volume_type_not_found_exception import VolumeTypeNotFoundException
from almanach.common.exceptions.almanach_exception import AlmanachException
from almanach import config
from almanach.core.model import todict
from tests.builder import a, instance, volume, volume_type


class DatabaseAdapterTest(unittest.TestCase):
    def setUp(self):
        config.read(pkg_resources.resource_filename("almanach", "resources/config/test.cfg"))
        mongo_connection = mongomock.Connection()

        self.adapter = DatabaseAdapter()
        self.db = mongo_connection[config.mongodb_database()]

        flexmock(MongoClient).new_instances(mongo_connection)

    def tearDown(self):
        flexmock_teardown()

    def test_insert_instance(self):
        fake_instance = a(instance())
        self.adapter.insert_entity(fake_instance)

        self.assertEqual(self.db.entity.count(), 1)
        self.assert_mongo_collection_contains("entity", fake_instance)

    def test_has_active_entity_not_found(self):
        self.assertFalse(self.adapter.has_active_entity("my_entity_id"))

    def test_has_active_entity_found(self):
        fake_instance = a(instance().with_id("my_entity_id"))
        self.adapter.insert_entity(fake_instance)
        self.assertTrue(self.adapter.has_active_entity("my_entity_id"))

    def test_get_instance_entity(self):
        fake_entity = a(instance().with_metadata({}))

        self.db.entity.insert(todict(fake_entity))

        self.assertEqual(self.adapter.get_active_entity(fake_entity.entity_id), fake_entity)

    def test_get_instance_entity_with_decode_output(self):
        fake_entity = a(instance().with_metadata({"a_metadata_not_sanitize": "not.sanitize",
                                                  "a_metadata^to_sanitize": "this.sanitize"}))

        self.db.entity.insert(todict(fake_entity))

        entity = self.adapter.get_active_entity(fake_entity.entity_id)

        expected_entity = a(instance()
                            .with_id(fake_entity.entity_id)
                            .with_project_id(fake_entity.project_id)
                            .with_metadata({"a_metadata_not_sanitize": "not.sanitize",
                                            "a_metadata.to_sanitize": "this.sanitize"}))

        self.assertEqual(entity, expected_entity)
        self.assert_entities_metadata_have_been_sanitize([entity])

    def test_get_instance_entity_will_not_found(self):
        with self.assertRaises(KeyError):
            self.adapter.get_active_entity("will_not_found")

    def test_get_instance_entity_with_unknown_type(self):
        fake_entity = a(instance())
        fake_entity.entity_type = "will_raise_excepion"

        self.db.entity.insert(todict(fake_entity))

        with self.assertRaises(NotImplementedError):
            self.adapter.get_active_entity(fake_entity.entity_id)

    def test_count_entities(self):
        fake_active_entities = [
            a(volume().with_id("id2").with_start(2014, 1, 1, 1, 0, 0).with_no_end()),
            a(instance().with_id("id3").with_start(2014, 1, 1, 8, 0, 0).with_no_end()),
        ]
        fake_inactive_entities = [
            a(instance().with_id("id1").with_start(2014, 1, 1, 7, 0, 0).with_end(2014, 1, 1, 8, 0, 0)),
            a(volume().with_id("id2").with_start(2014, 1, 1, 1, 0, 0).with_end(2014, 1, 1, 8, 0, 0)),
        ]
        [self.db.entity.insert(todict(fake_entity)) for fake_entity in fake_active_entities + fake_inactive_entities]

        self.assertEqual(4, self.adapter.count_entities())
        self.assertEqual(2, self.adapter.count_active_entities())
        self.assertEqual(1, self.adapter.count_entity_entries("id1"))
        self.assertEqual(2, self.adapter.count_entity_entries("id2"))

    def test_get_entity(self):
        fake_entity = a(instance().with_id("id1").with_start(2014, 1, 1, 8, 0, 0).with_no_end())
        self.db.entity.insert(todict(fake_entity))

        entries = self.adapter.get_all_entities_by_id(entity_id="id1")
        self.assertEqual(1, len(entries))
        self.assertEqual("id1", entries[0].entity_id)

        entries = self.adapter.get_all_entities_by_id(entity_id="id2")
        self.assertEqual(0, len(entries))

    def test_list_instances(self):
        fake_instances = [
            a(instance().with_id("id1").with_start(2014, 1, 1, 7, 0, 0).with_end(
                2014, 1, 1, 8, 0, 0).with_project_id("project_id").with_metadata({})),
            a(instance().with_id("id2").with_start(2014, 1, 1, 1, 0,
                                                   0).with_no_end().with_project_id("project_id").with_metadata({})),
            a(instance().with_id("id3").with_start(2014, 1, 1, 8, 0,
                                                   0).with_no_end().with_project_id("project_id").with_metadata({})),
        ]
        fake_volumes = [
            a(volume().with_id("id1").with_start(2014, 1, 1, 7, 0, 0).with_end(
                2014, 1, 1, 8, 0, 0).with_project_id("project_id")),
            a(volume().with_id("id2").with_start(2014, 1, 1, 1, 0, 0).with_no_end().with_project_id("project_id")),
            a(volume().with_id("id3").with_start(2014, 1, 1, 8, 0, 0).with_no_end().with_project_id("project_id")),
        ]
        [self.db.entity.insert(todict(fake_entity)) for fake_entity in fake_instances + fake_volumes]

        entities = self.adapter.list_entities("project_id", datetime(
            2014, 1, 1, 0, 0, 0, tzinfo=pytz.utc), datetime(2014, 1, 1, 12, 0, 0, tzinfo=pytz.utc), "instance")
        assert_that(entities, contains_inanyorder(*fake_instances))

    def test_list_instances_with_decode_output(self):
        fake_instances = [
            a(instance()
              .with_id("id1")
              .with_start(2014, 1, 1, 7, 0, 0)
              .with_end(2014, 1, 1, 8, 0, 0)
              .with_project_id("project_id")
              .with_metadata({"a_metadata_not_sanitize": "not.sanitize",
                              "a_metadata^to_sanitize": "this.sanitize"})),
            a(instance()
              .with_id("id2")
              .with_start(2014, 1, 1, 1, 0, 0)
              .with_no_end()
              .with_project_id("project_id")
              .with_metadata({"a_metadata^to_sanitize": "this.sanitize"})),
        ]

        expected_instances = [
            a(instance()
              .with_id("id1")
              .with_start(2014, 1, 1, 7, 0, 0)
              .with_end(2014, 1, 1, 8, 0, 0)
              .with_project_id("project_id")
              .with_metadata({"a_metadata_not_sanitize": "not.sanitize",
                              "a_metadata.to_sanitize": "this.sanitize"})),
            a(instance()
              .with_id("id2")
              .with_start(2014, 1, 1, 1, 0, 0)
              .with_no_end()
              .with_project_id("project_id")
              .with_metadata({"a_metadata.to_sanitize": "this.sanitize"})),
        ]

        [self.db.entity.insert(todict(fake_entity)) for fake_entity in fake_instances]

        entities = self.adapter.list_entities("project_id", datetime(
            2014, 1, 1, 0, 0, 0, tzinfo=pytz.utc), datetime(2014, 1, 1, 12, 0, 0, tzinfo=pytz.utc), "instance")
        assert_that(entities, contains_inanyorder(*expected_instances))
        self.assert_entities_metadata_have_been_sanitize(entities)

    def test_list_entities_in_period(self):
        fake_entities_in_period = [
            a(instance().with_id("in_the_period").with_start(2014, 1, 1, 7, 0,
                                                             0).with_end(2014, 1, 1, 8, 0, 0).with_project_id(
                "project_id")),
            a(instance().with_id("running_has_started_before").with_start(
                2014, 1, 1, 1, 0, 0).with_no_end().with_project_id("project_id")),
            a(instance().with_id("running_has_started_during").with_start(
                2014, 1, 1, 8, 0, 0).with_no_end().with_project_id("project_id")),
        ]
        fake_entities_out_period = [
            a(instance().with_id("before_the_period").with_start(2014, 1, 1, 0,
                                                                 0, 0).with_end(2014, 1, 1, 1, 0, 0).with_project_id(
                "project_id")),
            a(instance().with_id("after_the_period").with_start(2014, 1, 1, 10,
                                                                0, 0).with_end(2014, 1, 1, 11, 0, 0).with_project_id(
                "project_id")),
            a(instance().with_id("running_has_started_after").with_start(
                2014, 1, 1, 10, 0, 0).with_no_end().with_project_id("project_id")),
        ]
        [self.db.entity.insert(todict(fake_entity))
         for fake_entity in fake_entities_in_period + fake_entities_out_period]

        entities = self.adapter.list_entities("project_id", datetime(
            2014, 1, 1, 6, 0, 0, tzinfo=pytz.utc), datetime(2014, 1, 1, 9, 0, 0, tzinfo=pytz.utc))
        assert_that(entities, contains_inanyorder(*fake_entities_in_period))

    def test_list_entities_by_id(self):
        start = datetime(2016, 3, 1, 0, 0, 0, 0, pytz.utc)
        end = datetime(2016, 3, 3, 0, 0, 0, 0, pytz.utc)
        proper_instance = a(instance().with_id("id1").with_start(2016, 3, 1, 0, 0, 0).with_end(2016, 3, 2, 0, 0, 0))
        instances = [
            proper_instance,
            a(instance()
              .with_id("id1")
              .with_start(2016, 3, 2, 0, 0, 0)
              .with_no_end()),
        ]
        [self.db.entity.insert(todict(fake_instance)) for fake_instance in instances]

        instance_list = self.adapter.list_entities_by_id("id1", start, end)

        assert_that(instance_list, contains_inanyorder(*[proper_instance]))

    def test_update_active_entity(self):
        fake_entity = a(instance())
        end_date = datetime(2015, 10, 21, 16, 29, 0)

        self.db.entity.insert(todict(fake_entity))
        self.adapter.close_active_entity(fake_entity.entity_id, end_date)

        self.assertEqual(self.db.entity.find_one({"entity_id": fake_entity.entity_id})["end"], end_date)

    def test_update_closed_entity(self):
        fake_entity = a(instance().with_end(2016, 3, 2, 0, 0, 0))

        self.db.entity.insert(todict(fake_entity))
        fake_entity.flavor = "my_new_flavor"
        self.adapter.update_closed_entity(fake_entity, data={"flavor": fake_entity.flavor})

        db_entity = self.db.entity.find_one({"entity_id": fake_entity.entity_id})
        assert_that(db_entity['flavor'], fake_entity.flavor)
        assert_that(db_entity['end'], fake_entity.end)

    def test_replace_entity(self):
        fake_entity = a(instance())
        fake_entity.os.distro = "Centos"

        self.db.entity.insert(todict(fake_entity))
        fake_entity.os.distro = "Windows"

        self.adapter.update_active_entity(fake_entity)

        self.assertEqual(self.db.entity.find_one({"entity_id": fake_entity.entity_id})[
                             "os"]["distro"], fake_entity.os.distro)

    def test_insert_volume(self):
        count = self.db.entity.count()
        fake_volume = a(volume())
        self.adapter.insert_entity(fake_volume)

        self.assertEqual(count + 1, self.db.entity.count())
        self.assert_mongo_collection_contains("entity", fake_volume)

    def test_delete_active_entity(self):
        fake_entity = a(volume())

        self.db.entity.insert(todict(fake_entity))
        self.assertEqual(1, self.db.entity.count())

        self.adapter.delete_active_entity(fake_entity.entity_id)
        self.assertEqual(0, self.db.entity.count())

    def test_insert_volume_type(self):
        fake_volume_type = a(volume_type())
        self.adapter.insert_volume_type(fake_volume_type)

        self.assertEqual(1, self.db.volume_type.count())
        self.assert_mongo_collection_contains("volume_type", fake_volume_type)

    def test_get_volume_type(self):
        fake_volume_type = a(volume_type())
        self.db.volume_type.insert(todict(fake_volume_type))
        self.assertEqual(self.adapter.get_volume_type(fake_volume_type.volume_type_id), fake_volume_type)

    def test_get_volume_type_not_exist(self):
        fake_volume_type = a(volume_type())

        with self.assertRaises(VolumeTypeNotFoundException):
            self.adapter.get_volume_type(fake_volume_type.volume_type_id)

    def test_delete_volume_type(self):
        fake_volume_type = a(volume_type())
        self.db.volume_type.insert(todict(fake_volume_type))
        self.assertEqual(1, self.db.volume_type.count())
        self.adapter.delete_volume_type(fake_volume_type.volume_type_id)
        self.assertEqual(0, self.db.volume_type.count())

    def test_delete_volume_type_not_in_database(self):
        with self.assertRaises(AlmanachException):
            self.adapter.delete_volume_type("not_in_database_id")

    def test_delete_all_volume_types_not_permitted(self):
        with self.assertRaises(AlmanachException):
            self.adapter.delete_volume_type(None)

    def test_list_volume_types(self):
        fake_volume_types = [a(volume_type()), a(volume_type())]

        for fake_volume_type in fake_volume_types:
            self.db.volume_type.insert(todict(fake_volume_type))

        self.assertEqual(len(self.adapter.list_volume_types()), 2)

    def assert_mongo_collection_contains(self, collection, obj):
        (self.assertTrue(obj.as_dict() in self.db[collection].find(fields={"_id": 0}),
                         "The collection '%s' does not contains the object of type '%s'" % (collection, type(obj))))

    def assert_entities_metadata_have_been_sanitize(self, entities):
        for entity in entities:
            for key in entity.metadata:
                self.assertTrue(key.find("^") == -1,
                                "The metadata key %s contains carret" % (key))



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from hamcrest import assert_that, equal_to, has_entries, has_length, has_key, has_entry

from almanach.common.exceptions.almanach_exception import AlmanachException
from tests.builder import volume_type, a
from tests.api.base_api import BaseApi


class ApiVolumeTypeTest(BaseApi):

    def test_get_volume_types(self):
        self.controller.should_receive('list_volume_types') \
            .and_return([a(volume_type().with_volume_type_name('some_volume_type_name'))]) \
            .once()

        code, result = self.api_get('/volume_types', headers={'X-Auth-Token': 'some token value'})

        assert_that(code, equal_to(200))
        assert_that(result, has_length(1))
        assert_that(result[0], has_key('volume_type_name'))
        assert_that(result[0]['volume_type_name'], equal_to('some_volume_type_name'))

    def test_successful_volume_type_create(self):
        data = dict(
                type_id='A_VOLUME_TYPE_ID',
                type_name="A_VOLUME_TYPE_NAME"
        )

        self.controller.should_receive('create_volume_type') \
            .with_args(
                volume_type_id=data['type_id'],
                volume_type_name=data['type_name']) \
            .once()

        code, result = self.api_post('/volume_type', data=data, headers={'X-Auth-Token': 'some token value'})
        assert_that(code, equal_to(201))

    def test_volume_type_create_missing_a_param_returns_bad_request_code(self):
        data = dict(type_name="A_VOLUME_TYPE_NAME")

        self.controller.should_receive('create_volume_type') \
            .never()

        code, result = self.api_post('/volume_type', data=data, headers={'X-Auth-Token': 'some token value'})
        assert_that(code, equal_to(400))
        assert_that(result, has_entries({"error": "The 'type_id' param is mandatory for the request you have made."}))

    def test_volume_type_delete_with_authentication(self):
        self.controller.should_receive('delete_volume_type') \
            .with_args('A_VOLUME_TYPE_ID') \
            .once()

        code, result = self.api_delete('/volume_type/A_VOLUME_TYPE_ID', headers={'X-Auth-Token': 'some token value'})
        assert_that(code, equal_to(202))

    def test_volume_type_delete_not_in_database(self):
        self.controller.should_receive('delete_volume_type') \
            .with_args('A_VOLUME_TYPE_ID') \
            .and_raise(AlmanachException("An exception occurred")) \
            .once()

        code, result = self.api_delete('/volume_type/A_VOLUME_TYPE_ID', headers={'X-Auth-Token': 'some token value'})

        assert_that(code, equal_to(500))
        assert_that(result, has_entry("error", "An exception occurred"))



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import os
import requests


class AlmanachHelper(object):
    x_auth_token = 'secret'

    def __init__(self):
        is_container = True if os.environ.get('TEST_CONTAINER') else False
        port = 8000 if is_container else 80
        hostname = "api" if is_container else "127.0.0.1"
        self.base_url = "http://{url}:{port}".format(url=hostname, port=port)

    def get_entities(self, tenant_id, start):
        url = "{url}/project/{project}/entities?start={start}".format(
            url=self.base_url, project=tenant_id, start=start
        )

        response = requests.get(url, headers=self._get_query_headers())
        return response.json()

    def get(self, url, headers=None, **params):
        return requests.get(
            url.format(url=self.base_url, **params),
            headers=headers if headers else self._get_query_headers()
        )

    def head(self, url, headers=None, **params):
        return requests.head(
            url.format(url=self.base_url, **params),
            headers=headers if headers else self._get_query_headers()
        )

    def post(self, url, data, **params):
        return requests.post(
            url.format(url=self.base_url, **params),
            data=json.dumps(data),
            headers=self._get_query_headers()
        )

    def put(self, url, data, **params):
        return requests.put(
            url.format(url=self.base_url, **params),
            data=json.dumps(data),
            headers=self._get_query_headers()
        )

    def delete(self, url, data, **params):
        return requests.delete(
            url.format(url=self.base_url, **params),
            data=json.dumps(data),
            headers=self._get_query_headers()
        )

    def _get_query_headers(self):
        return {
            'X-Auth-Token': self.x_auth_token,
            'Accept': 'application/json'
        }



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class VolumeBusAdapter(object):
    def __init__(self, controller):
        self.controller = controller

    def handle_events(self, event_type, notification):
        if event_type == "volume.create.end":
            self.on_volume_created(notification)
        elif event_type == "volume.delete.end":
            self.on_volume_deleted(notification)
        elif event_type == "volume.resize.end":
            self.on_volume_resized(notification)
        elif event_type == "volume.attach.end":
            self.on_volume_attached(notification)
        elif event_type == "volume.detach.end":
            self.on_volume_detached(notification)
        elif event_type == "volume.update.end":
            self.on_volume_renamed(notification)
        elif event_type == "volume.exists":
            self.on_volume_renamed(notification)
        elif event_type == "volume_type.create":
            self.on_volume_type_create(notification)

    def on_volume_created(self, notification):
        payload = notification.get("payload")
        date = payload.get("created_at")
        project_id = payload.get("tenant_id")
        volume_id = payload.get("volume_id")
        volume_name = payload.get("display_name")
        volume_type = payload.get("volume_type")
        volume_size = payload.get("size")
        self.controller.create_volume(volume_id, project_id, date, volume_type, volume_size, volume_name)

    def on_volume_deleted(self, notification):
        payload = notification.get("payload")
        volume_id = payload.get("volume_id")
        end_date = notification.get("timestamp")
        self.controller.delete_volume(volume_id, end_date)

    def on_volume_renamed(self, notification):
        payload = notification.get("payload")
        volume_id = payload.get("volume_id")
        volume_name = payload.get("display_name")
        self.controller.rename_volume(volume_id, volume_name)

    def on_volume_resized(self, notification):
        payload = notification.get("payload")
        date = notification.get("timestamp")
        volume_id = payload.get("volume_id")
        volume_size = payload.get("size")
        self.controller.resize_volume(volume_id, volume_size, date)

    def on_volume_attached(self, notification):
        payload = notification.get("payload")
        volume_id = payload.get("volume_id")
        event_date = notification.get("timestamp")
        self.controller.attach_volume(volume_id, event_date, self._get_attached_instances(payload))

    def on_volume_detached(self, notification):
        payload = notification.get("payload")
        volume_id = payload.get("volume_id")
        event_date = notification.get("timestamp")
        self.controller.detach_volume(volume_id, event_date, self._get_attached_instances(payload))

    def on_volume_type_create(self, notification):
        volume_types = notification.get("payload").get("volume_types")
        volume_type_id = volume_types.get("id")
        volume_type_name = volume_types.get("name")
        self.controller.create_volume_type(volume_type_id, volume_type_name)

    @staticmethod
    def _get_attached_instances(payload):
        instances_ids = []
        if "volume_attachment" in payload:
            for instance in payload["volume_attachment"]:
                instances_ids.append(instance.get("instance_uuid"))
        elif payload.get("instance_uuid") is not None:
            instances_ids.append(payload.get("instance_uuid"))
        return instances_ids



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

import pymongo
from pymongo.errors import ConfigurationError
from pymongomodem.utils import decode_output, encode_input

from almanach import config
from almanach.common.exceptions.almanach_exception import AlmanachException
from almanach.common.exceptions.volume_type_not_found_exception import VolumeTypeNotFoundException
from almanach.core.model import build_entity_from_dict, VolumeType


def database(function):
    def _connection(self, *args, **kwargs):
        try:
            if not self.db:
                connection = pymongo.MongoClient(config.mongodb_url(), tz_aware=True)
                self.db = connection[config.mongodb_database()]
                ensureindex(self.db)
            return function(self, *args, **kwargs)
        except KeyError as e:
            raise e
        except VolumeTypeNotFoundException as e:
            raise e
        except NotImplementedError as e:
            raise e
        except ConfigurationError as e:
            logging.exception("DB Connection, make sure username and password doesn't contain the following :+&/ "
                              "character")
            raise e
        except Exception as e:
            logging.exception(e)
            raise e

    return _connection


def ensureindex(db):
    db.entity.ensure_index(
        [(index, pymongo.ASCENDING)
         for index in config.mongodb_indexes()])


class DatabaseAdapter(object):
    def __init__(self):
        self.db = None

    @database
    def get_active_entity(self, entity_id):
        entity = self._get_one_entity_from_db({"entity_id": entity_id, "end": None})
        if not entity:
            raise KeyError("Unable to find entity id %s" % entity_id)
        return build_entity_from_dict(entity)

    @database
    def count_entities(self):
        return self.db.entity.count()

    @database
    def count_active_entities(self):
        return self.db.entity.find({"end": None}).count()

    @database
    def count_entity_entries(self, entity_id):
        return self.db.entity.find({"entity_id": entity_id}).count()

    @database
    def has_active_entity(self, entity_id):
        return self.db.entity.find({"entity_id": entity_id, "end": None}).count() == 1

    @database
    def list_entities(self, project_id, start, end, entity_type=None):
        args = {"project_id": project_id, "start": {"$lte": end}, "$or": [{"end": None}, {"end": {"$gte": start}}]}
        if entity_type:
            args["entity_type"] = entity_type
        entities = self._get_entities_from_db(args)
        return [build_entity_from_dict(entity) for entity in entities]

    @database
    def get_all_entities_by_id(self, entity_id):
        entities = self.db.entity.find({"entity_id": entity_id}, {"_id": 0})
        return [build_entity_from_dict(entity) for entity in entities]

    @database
    def list_entities_by_id(self, entity_id, start, end):
        entities = self.db.entity.find({"entity_id": entity_id,
                                        "start": {"$gte": start},
                                        "$and": [
                                            {"end": {"$ne": None}},
                                            {"end": {"$lte": end}}
                                            ]
                                        }, {"_id": 0})
        return [build_entity_from_dict(entity) for entity in entities]

    @database
    def update_closed_entity(self, entity, data):
        self.db.entity.update({"entity_id": entity.entity_id, "start": entity.start, "end": entity.end},
                              {"$set": data})

    @database
    def insert_entity(self, entity):
        self._insert_entity(entity.as_dict())

    @database
    def insert_volume_type(self, volume_type):
        self.db.volume_type.insert(volume_type.__dict__)

    @database
    def get_volume_type(self, volume_type_id):
        volume_type = self.db.volume_type.find_one({"volume_type_id": volume_type_id})
        if not volume_type:
            logging.error("Trying to get a volume type not in the database.")
            raise VolumeTypeNotFoundException(volume_type_id=volume_type_id)

        return VolumeType(volume_type_id=volume_type["volume_type_id"],
                          volume_type_name=volume_type["volume_type_name"])

    @database
    def delete_volume_type(self, volume_type_id):
        if volume_type_id is None:
            error = "Trying to delete all volume types which is not permitted."
            logging.error(error)
            raise AlmanachException(error)
        returned_value = self.db.volume_type.remove({"volume_type_id": volume_type_id})
        if returned_value['n'] == 1:
            logging.info("Deleted volume type with id '%s' successfully." % volume_type_id)
        else:
            error = "Volume type with id '%s' doesn't exist in the database." % volume_type_id
            logging.error(error)
            raise AlmanachException(error)

    @database
    def list_volume_types(self):
        volume_types = self.db.volume_type.find()
        return [VolumeType(volume_type_id=volume_type["volume_type_id"],
                           volume_type_name=volume_type["volume_type_name"]) for volume_type in volume_types]

    @database
    def close_active_entity(self, entity_id, end):
        self.db.entity.update({"entity_id": entity_id, "end": None}, {"$set": {"end": end, "last_event": end}})

    @database
    def update_active_entity(self, entity):
        self.db.entity.update({"entity_id": entity.entity_id, "end": None}, {"$set": entity.as_dict()})

    @database
    def delete_active_entity(self, entity_id):
        self.db.entity.remove({"entity_id": entity_id, "end": None})

    @encode_input
    def _insert_entity(self, entity):
        self.db.entity.insert(entity)

    @decode_output
    def _get_entities_from_db(self, args):
        return list(self.db.entity.find(args, {"_id": 0}))

    @decode_output
    def _get_one_entity_from_db(self, args):
        return self.db.entity.find_one(args, {"_id": 0})



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from hamcrest import assert_that, equal_to

from tests.api.base_api import BaseApi


class ApiAuthenticationTest(BaseApi):
    def setUp(self):
        self.prepare()
        self.prepare_with_failed_authentication()

    def test_with_wrong_authentication(self):
        self.controller.should_receive('list_entities').never()
        query_string = {'start': '2014-01-01 00:00:00.0000', 'end': '2014-02-01 00:00:00.0000'}

        code, result = self.api_get(url='/project/TENANT_ID/entities',
                                    query_string=query_string,
                                    headers={'X-Auth-Token': 'wrong token'})

        assert_that(code, equal_to(401))



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import sys
import logging
import logging.config as logging_config

from almanach import config
from almanach.api import AlmanachApi
from almanach.collector import AlmanachCollector


def run():
    parser = argparse.ArgumentParser()
    parser.add_argument("service", help="Service to execute: 'api' or 'collector'", choices=["api", "collector"])
    parser.add_argument("config_file", help="Config file path")
    parser.add_argument("--logging", help="Logger configuration")
    parser.add_argument("--port", help="API HTTP port (default is 8000)", default=8000)
    parser.add_argument("--host", help="API hostname to listen on (default is 127.0.0.1)", default="127.0.0.1")
    args = parser.parse_args()

    config.read(args.config_file)

    if args.logging:
        print("Loading logger configuration from {0}".format(args.logging))
        logging_config.fileConfig(args.logging, disable_existing_loggers=False)
    else:
        logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
        logging.debug("Logging to stdout")

    if args.service == "api":
        almanach_api = AlmanachApi()
        almanach_api.run(host=args.host, port=args.port)
    else:
        almanach_collector = AlmanachCollector()
        almanach_collector.run()


if __name__ == "__main__":
    run()



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytz
import unittest
from datetime import datetime
from flexmock import flexmock, flexmock_teardown

from almanach.adapters.bus_adapter import BusAdapter
from almanach.common.exceptions.almanach_entity_not_found_exception import AlmanachEntityNotFoundException
from integration_tests.builders import messages


class BusAdapterTest(unittest.TestCase):

    def setUp(self):
        self.controller = flexmock()
        self.retry = flexmock()
        self.bus_adapter = BusAdapter(self.controller, None, retry_adapter=self.retry)

    def tearDown(self):
        flexmock_teardown()

    def test_on_message(self):
        instance_id = "e7d44dea-21c1-452c-b50c-cbab0d07d7d3"
        tenant_id = "0be9215b503b43279ae585d50a33aed8"
        instance_type = "myflavor"
        timestamp = datetime(2014, 02, 14, 16, 30, 10, tzinfo=pytz.utc)
        hostname = "some hostname"
        metadata = {"a_metadata.to_filter": "filtered_value", }

        notification = messages.get_instance_create_end_sample(instance_id=instance_id, tenant_id=tenant_id,
                                                               flavor_name=instance_type, creation_timestamp=timestamp,
                                                               name=hostname, metadata=metadata)
        os_type = notification.get("payload").get("image_meta").get("os_type")
        distro = notification.get("payload").get("image_meta").get("distro")
        version = notification.get("payload").get("image_meta").get("version")
        metadata = notification.get("payload").get("metadata")

        self.controller \
            .should_receive("create_instance") \
            .with_args(
                instance_id,
                tenant_id,
                timestamp.strftime("%Y-%m-%dT%H:%M:%S.%fZ"),
                instance_type,
                os_type,
                distro,
                version,
                hostname,
                metadata
            ) \
            .once()

        message = flexmock()
        message.should_receive("ack")

        self.bus_adapter.on_message(notification, message)

    def test_on_message_with_empty_metadata(self):
        instance_id = "e7d44dea-21c1-452c-b50c-cbab0d07d7d3"
        tenant_id = "0be9215b503b43279ae585d50a33aed8"
        instance_type = "myflavor"
        timestamp = datetime(2014, 02, 14, 16, 30, 10, tzinfo=pytz.utc)
        hostname = "some hostname"

        notification = messages.get_instance_create_end_sample(instance_id=instance_id, tenant_id=tenant_id,
                                                               flavor_name=instance_type, creation_timestamp=timestamp,
                                                               name=hostname, metadata={})
        os_type = notification.get("payload").get("image_meta").get("os_type")
        distro = notification.get("payload").get("image_meta").get("distro")
        version = notification.get("payload").get("image_meta").get("version")
        metadata = notification.get("payload").get("metadata")

        self.controller \
            .should_receive("create_instance") \
            .with_args(
                instance_id, tenant_id, timestamp.strftime("%Y-%m-%dT%H:%M:%S.%fZ"), instance_type, os_type,
                distro, version, hostname, metadata
            ) \
            .once()

        message = flexmock()
        (flexmock(message)
         .should_receive("ack"))

        self.bus_adapter.on_message(notification, message)

    def test_on_message_with_delete_instance(self):
        notification = messages.get_instance_delete_end_sample()

        self.controller \
            .should_receive("delete_instance") \
            .with_args(
                notification['payload']['instance_id'],
                notification['payload']['terminated_at']
            ) \
            .once()

        message = flexmock()
        (flexmock(message)
         .should_receive("ack"))

        self.bus_adapter.on_message(notification, message)

    def test_on_message_with_rebuild_instance(self):
        notification = messages.get_instance_rebuild_end_sample()

        self.controller \
            .should_receive("rebuild_instance") \
            .with_args(
                notification['payload']['instance_id'],
                notification['payload']['image_meta']['distro'],
                notification['payload']['image_meta']['version'],
                notification['payload']['image_meta']['os_type'],
                notification['timestamp'],
            ) \
            .once()

        message = flexmock()
        (flexmock(message)
         .should_receive("ack"))

        self.bus_adapter.on_message(notification, message)

    def test_on_message_with_resize_instance(self):
        notification = messages.get_instance_resized_end_sample()

        self.controller \
            .should_receive("resize_instance") \
            .with_args(
                notification['payload']['instance_id'],
                notification['payload']['instance_type'],
                notification['timestamp'],
            )\
            .once()

        message = flexmock()
        (flexmock(message)
         .should_receive("ack"))

        self.bus_adapter.on_message(notification, message)

    def test_on_message_with_resize_volume(self):
        notification = messages.get_volume_update_end_sample()

        self.controller \
            .should_receive("resize_volume") \
            .with_args(
                notification['payload']['volume_id'],
                notification['payload']['size'],
                notification['timestamp'],
            ) \
            .once()

        message = flexmock()
        (flexmock(message)
         .should_receive("ack"))

        self.bus_adapter.on_message(notification, message)

    def test_on_message_with_volume(self):
        volume_id = "vol_id"
        tenant_id = "tenant_id"
        timestamp_datetime = datetime(2014, 02, 14, 16, 30, 10, tzinfo=pytz.utc)
        volume_type = "SF400"
        volume_size = 100000
        some_volume = "volume_name"

        notification = messages.get_volume_create_end_sample(volume_id=volume_id, tenant_id=tenant_id,
                                                             volume_type=volume_type, volume_size=volume_size,
                                                             creation_timestamp=timestamp_datetime, name=some_volume)
        self.controller \
            .should_receive("create_volume") \
            .with_args(
                volume_id, tenant_id, timestamp_datetime.strftime("%Y-%m-%dT%H:%M:%S.%fZ"), volume_type,
                volume_size, some_volume
            ) \
            .once()

        message = flexmock()
        (flexmock(message)
         .should_receive("ack"))

        self.bus_adapter.on_message(notification, message)

    def test_on_message_with_volume_type(self):
        volume_type_id = "an_id"
        volume_type_name = "a_name"

        notification = messages.get_volume_type_create_sample(volume_type_id=volume_type_id,
                                                              volume_type_name=volume_type_name)

        self.controller \
            .should_receive("create_volume_type") \
            .with_args(volume_type_id, volume_type_name) \
            .once()

        message = flexmock()
        (flexmock(message)
         .should_receive("ack"))

        self.bus_adapter.on_message(notification, message)

    def test_on_message_with_delete_volume(self):
        notification = messages.get_volume_delete_end_sample()

        self.controller \
            .should_receive("delete_volume") \
            .with_args(
                notification['payload']['volume_id'],
                notification['timestamp'],
            ) \
            .once()

        message = flexmock()
        (flexmock(message)
         .should_receive("ack"))

        self.bus_adapter.on_message(notification, message)

    def test_failing_notification_get_retry(self):
        notification = messages.get_instance_rebuild_end_sample()
        message = flexmock()

        (flexmock(message).should_receive("ack"))
        self.controller.should_receive('instance_rebuilded').and_raise(Exception("Foobar"))
        self.retry.should_receive('publish_to_dead_letter').with_args(message).once()

        self.bus_adapter.on_message(notification, message)

    def test_that_entity_not_found_exceptions_goes_to_retry_queue(self):
        notification = messages.get_instance_delete_end_sample(instance_id="My instance id")
        message = flexmock()

        (flexmock(message).should_receive("ack"))
        self.controller.should_receive('delete_instance').and_raise(AlmanachEntityNotFoundException("Entity not found"))
        self.retry.should_receive('publish_to_dead_letter').with_args(message).once()

        self.bus_adapter.on_message(notification, message)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from uuid import uuid4
from hamcrest import assert_that, equal_to

from base_api_volume_testcase import BaseApiVolumeTestCase
from builders import messages


class ApiVolumeTypeTest(BaseApiVolumeTestCase):
    def test_volume_type_create(self):
        volume_type_query = "{url}/volume_type"
        volume_type_id = str(uuid4())
        data = dict(
                type_id=volume_type_id,
                type_name=messages.DEFAULT_VOLUME_NAME
        )

        response = self.almanachHelper.post(url=volume_type_query, data=data)
        assert_that(response.status_code, equal_to(201))

        volume_type_get_query = "{url}/volume_type/{volume_type_id}"

        response = self.almanachHelper.get(url=volume_type_get_query, volume_type_id=volume_type_id)
        assert_that(response.status_code, equal_to(200))
        assert_that(response.json(), equal_to({'volume_type_id': data['type_id'],
                                               'volume_type_name': data['type_name']}))
